@@ -442,9 +442,11 @@ static int parse_iic(FILE *image, void *context,
 	if (initial_pos < 0)
 		return -1;
 
-	fseek(image, 0L, SEEK_END);
+	if (fseek(image, 0L, SEEK_END) != 0)
+		return -1;
 	file_size = ftell(image);
-	fseek(image, initial_pos, SEEK_SET);
+	if (fseek(image, initial_pos, SEEK_SET) != 0)
+		return -1;
 	for (;;) {
 		/* Ignore the trailing reset IIC data (5 bytes) */
 		if (ftell(image) >= (file_size - 5))
@@ -633,7 +635,8 @@ static int fx3_load_ram(libusb_device_handle *device, const char *path)
 		if (dLength == 0)
 			break; // done
 
-		dImageBuf = calloc(dLength, sizeof(uint32_t));
+		// coverity[tainted_data]
+		dImageBuf = (uint32_t*)calloc(dLength, sizeof(uint32_t));
 		if (dImageBuf == NULL) {
 			logerror("could not allocate buffer for image chunk\n");
 			ret = -4;
@@ -813,9 +816,10 @@ int ezusb_load_ram(libusb_device_handle *device, const char *path, int fx_type,
 		}
 	}
 
-	if (verbose)
+	if (verbose && (ctx.count != 0)) {
 		logerror("... WROTE: %d bytes, %d segments, avg %d\n",
-		(int)ctx.total, (int)ctx.count, (int)(ctx.total/ctx.count));
+			(int)ctx.total, (int)ctx.count, (int)(ctx.total/ctx.count));
+	}
 
 	/* if required, reset the CPU so it runs what we just uploaded */
 	if (cpucs_addr && !ezusb_cpucs(device, cpucs_addr, true))
